@@ -5,15 +5,30 @@ import numpy as np
 
 from .abinitparser import AbinitParser
 from .elkparser import ElkParser
+from .qeparser import QEParser
+from .lobsterparser import LobsterParser
 from .procarparser import ProcarParser
 from .procarplot import ProcarPlot
 from .procarselect import ProcarSelect
 from .splash import welcome
 from .utilsprocar import UtilsProcar
 
+# import matplotlib
+plt.rcParams["mathtext.default"] = "regular"  # Roman ['rm', 'cal', 'it', 'tt', 'sf',
+#                                                   'bf', 'default', 'bb', 'frak',
+#                                                   'circled', 'scr', 'regular']
+plt.rcParams["font.family"] = "Georgia"
+plt.rc("font", size=22)  # controls default text sizes
+plt.rc("axes", titlesize=22)  # fontsize of the axes title
+plt.rc("axes", labelsize=22)  # fontsize of the x and y labels
+plt.rc("xtick", labelsize=22)  # fontsize of the tick labels
+plt.rc("ytick", labelsize=22)  # fontsize of the tick labels
+# plt.rc('legend', fontsize=22)    # legend fontsize
+# plt.rc('figure', titlesize=22)  # fontsize of the figure title
+
 
 def bandsplot(
-    file=None,
+    procarfile=None,
     mode="plain",
     color="blue",
     abinit_output=None,
@@ -21,7 +36,7 @@ def bandsplot(
     atoms=None,
     orbitals=None,
     fermi=None,
-    elimit=None,
+    elimit=[-2, 2],
     mask=None,
     markersize=0.02,
     cmap="jet",
@@ -37,11 +52,13 @@ def bandsplot(
     title=None,
     outcar=None,
     kpointsfile=None,
-    exportplt=False,
     kdirect=True,
-    discontinuities=[],
     code="vasp",
     separate=False,
+    ax=None,
+    discontinuities=None,
+    show=True,
+    lobstercode="qe",
 ):
 
     """This function plots band structures
@@ -51,6 +68,8 @@ def bandsplot(
     # Turn interactive plotting off
     plt.ioff()
 
+    # Verbose section
+
     # First handling the options, to get feedback to the user and check
     # that the input makes sense.
     # It is quite long
@@ -64,49 +83,63 @@ def bandsplot(
         orbitals = [-1]
 
     print("Script initiated")
-    print("code          : ", code)
-    print("input file    : ", file)
-    print("Mode          : ", mode)
-    print("spin comp.    : ", spin)
-    print("atoms list   : ", atoms)
-    print("orbs. list   : ", orbitals)
+    print("code           : ", code)
+    print("input file     : ", procarfile)
+    print("mode           : ", mode)
+    print("spin comp.     : ", spin)
+    print("atoms list     : ", atoms)
+    print("orbs. list     : ", orbitals)
 
-    if fermi is None and outcar is None and abinit_output is None and code != "elk":
+    if (
+        fermi is None
+        and outcar is None
+        and abinit_output is None
+        and (code != "elk" and code != "qe" and code != "lobster")
+    ):
         print(
-            "WARNING: Fermi Energy not set! Please set manually or provide output file."
+            "WARNING : Fermi Energy not set! Please set manually or provide output file and set code type."
         )
         fermi = 0
 
-    if fermi is None and code == "elk":
+    elif fermi is None and code == "elk":
         fermi = None
 
-    print("Fermi Energy   : ", fermi)
-    print("Energy range  : ", elimit)
+    elif fermi is None and code == "qe":
+        fermi = None
+
+    elif fermi is None and code == "lobster":
+        fermi = None
+
+    print("fermi energy   : ", fermi)
+    print("energy range   : ", elimit)
 
     if mask is not None:
-        print("masking thres.: ", mask)
+        print("masking thres. : ", mask)
 
-    print("Colormap      : ", cmap)
-    print("MarkerSize    : ", markersize)
-    print("Permissive    : ", permissive)
+    print("colormap       : ", cmap)
+    print("markersize     : ", markersize)
+    print("permissive     : ", permissive)
     if permissive:
-        print("INFO: Permissive flag is on! Be careful")
-    print("vmax          : ", vmax)
-    print("vmin          : ", vmin)
-    print("grid enabled  : ", grid)
+        print("INFO : Permissive flag is on! Be careful")
+    print("vmax           : ", vmax)
+    print("vmin           : ", vmin)
+    print("grid enabled   : ", grid)
     if human is not None:
-        print("human         : ", human)
-    print("Savefig       : ", savefig)
-    print("title         : ", title)
-    print("outcar        : ", outcar)
+        print("human          : ", human)
+    print("savefig        : ", savefig)
+    print("title          : ", title)
+    print("outcar         : ", outcar)
 
     if kdirect:
-        print("k-point coordinates        : reduced")
+        print("k-grid         :  reduced")
     else:
         print(
-            "k-point coordinates        : cartesian (Remember to supply an output file for this case to work.)"
+            "k-grid         :  cartesian (Remember to provide an output file for this case to work.)"
         )
 
+    if discontinuities is None:
+        discontinuities = []
+
     #### READING KPOINTS FILE IF PRESENT ####
 
     # If KPOINTS file is given:
@@ -130,7 +163,7 @@ def bandsplot(
                 icounter = icounter + 2
             else:
                 discont_indx.append(icounter)
-                knames.append(tick_labels[icounter] + "/" + tick_labels[icounter + 1])
+                knames.append(tick_labels[icounter] + "|" + tick_labels[icounter + 1])
                 icounter = icounter + 2
         knames.append(tick_labels[-1])
         discont_indx = list(dict.fromkeys(discont_indx))
@@ -159,21 +192,20 @@ def bandsplot(
             gridpoint = gridpoint + numgridpoints
             kticks.append(gridpoint - 1)
 
-        print("knames        : ", knames)
-        print("kticks        : ", kticks)
+        print("knames         : ", knames)
+        print("kticks         : ", kticks)
 
         # creating an array for discontunuity k-points. These are the indexes
         # of the discontinuity k-points.
-        discontinuities = []
         for k in discont_indx:
             discontinuities.append(kticks[int(k / 2) + 1])
         if discontinuities:
-            print("discontinuities :", discontinuities)
+            print("discont. list  : ", discontinuities)
 
     #### END OF KPOINTS FILE DEPENDENT SECTION ####
 
     # The spin argument should be a number (index of an array), or
-    #'st'. In the last case it will be handled separately (later)
+    # 'st'. In the last case it will be handled separately (later)
 
     spin = {"0": 0, "1": 1, "2": 2, "3": 3, "st": "st"}[str(spin)]
 
@@ -196,6 +228,34 @@ def bandsplot(
                 kticks = procarFile.kticks
                 knames = procarFile.knames
 
+    elif code == "qe":
+        # reciprocal lattice already taken care of
+        procarFile = QEParser(kdirect=kdirect)
+
+        # Retrieving knames and kticks from QE
+        if kticks is None and knames is None:
+            if procarFile.kticks and procarFile.knames:
+                kticks = procarFile.kticks
+                knames = procarFile.knames
+
+            # Retrieving discontinuities if present
+            if procarFile.discontinuities:
+                discontinuities = procarFile.discontinuities
+
+    elif code == "lobster":
+        # reciprocal lattice already taken care of
+        procarFile = LobsterParser(kdirect=kdirect, lobstercode=lobstercode)
+
+        # Retrieving knames and kticks from Lobster
+        if kticks is None and knames is None:
+            if procarFile.kticks and procarFile.knames:
+                kticks = procarFile.kticks
+                knames = procarFile.knames
+
+            # Retrieving discontinuities if present
+            if procarFile.discontinuities:
+                discontinuities = procarFile.discontinuities
+
     # If ticks and names are given by the user manually:
     if kticks is not None and knames is not None:
         ticks = list(zip(kticks, knames))
@@ -205,10 +265,10 @@ def bandsplot(
         ticks = None
 
     if kpointsfile is None and kticks and knames:
-        print("kticks        : ", kticks)
-        print("knames        : ", knames)
+        print("knames         : ", knames)
+        print("kticks         : ", kticks)
         if discontinuities:
-            print("discontinuities :", discontinuities)
+            print("discont. list  : ", discontinuities)
 
     # The second part of this function is parse/select/use the data in
     # OUTCAR (if given) and PROCAR
@@ -220,18 +280,30 @@ def bandsplot(
             outcarparser = UtilsProcar()
             if fermi is None:
                 fermi = outcarparser.FermiOutcar(outcar)
-                print("Fermi energy found in OUTCAR file = " + str(fermi))
+                print("Fermi energy   :  %s eV (from OUTCAR)" % str(fermi))
             recLat = outcarparser.RecLatOutcar(outcar)
 
     elif code == "elk":
         if fermi is None:
             fermi = procarFile.fermi
-            print("Fermi energy found in Elk output file = " + str(fermi))
+            print("Fermi energy    :  %s eV (from Elk output)" % str(fermi))
+
+    elif code == "qe":
+        if fermi is None:
+            fermi = procarFile.fermi
+            print("Fermi energy   :  %s eV (from Quantum Espresso output)" % str(fermi))
+
+    elif code == "lobster":
+        if fermi is None:
+            fermi = procarFile.fermi
+            print("Fermi energy   :  %s eV (from Lobster output)" % str(fermi))
+            # lobster already shifts fermi so we set it to zero here.
+            fermi = 0.0
 
     elif code == "abinit":
         if fermi is None:
             fermi = abinitFile.fermi
-            print("Fermi energy found in Abinit output file = " + str(fermi))
+            print("Fermi energy   :  %s eV (from Abinit output)" % str(fermi))
         recLat = abinitFile.reclat
 
     # if kdirect = False, then the k-points will be in cartesian coordinates.
@@ -240,15 +312,15 @@ def bandsplot(
 
     if code == "vasp":
         if kdirect:
-            procarFile.readFile(file, permissive)
+            procarFile.readFile(procarfile, permissive)
         else:
-            procarFile.readFile(file, permissive, recLattice=recLat)
+            procarFile.readFile(procarfile, permissive, recLattice=recLat)
 
     elif code == "abinit":
         if kdirect:
-            procarFile.readFile(file, permissive)
+            procarFile.readFile(procarfile, permissive)
         else:
-            procarFile.readFile(file, permissive, recLattice=recLat)
+            procarFile.readFile(procarfile, permissive, recLattice=recLat)
 
     # processing the data, getting an instance of the class that reduces the data
     data = ProcarSelect(procarFile, deepCopy=True, mode=mode)
@@ -273,7 +345,7 @@ def bandsplot(
         cos = np.cos(angle)
         sin.shape = (sin.shape[0], 1)
         cos.shape = (cos.shape[0], 1)
-        ##print sin, cos
+        # print sin, cos
         # storing the spin projection into the original array
         data.spd = -sin * dataX.spd + cos * dataY.spd
     else:
@@ -282,7 +354,7 @@ def bandsplot(
         data.selectOrbital(orbitals)
 
     # Plotting the data
-    if separate == True:
+    if separate:
         if spin == 0:
             # plotting spin up bands separately
             data.bands = (
@@ -307,7 +379,7 @@ def bandsplot(
     ###### start of mode dependent options #########
 
     if mode == "scatter":
-        plot.scatterPlot(
+        fig, ax1 = plot.scatterPlot(
             mask=mask,
             size=markersize,
             cmap=cmap,
@@ -316,70 +388,75 @@ def bandsplot(
             marker=marker,
             ticks=ticks,
             discontinuities=discontinuities,
+            ax=ax,
         )
         if fermi is not None:
-            plt.ylabel(r"$E-E_f$ [eV]", fontsize=22)
+            ax1.set_ylabel(r"$E-E_f$ [eV]")
         else:
-            plt.ylabel(r"Energy [eV]", fontsize=22)
+            ax1.set_ylabel(r"Energy [eV]")
         if elimit is not None:
-            plt.ylim(elimit)
+            ax1.set_ylim(elimit)
 
     elif mode == "plain":
-        plot.plotBands(
+        fig, ax1 = plot.plotBands(
             markersize,
             marker=marker,
             ticks=ticks,
             color=color,
             discontinuities=discontinuities,
+            ax=ax,
         )
         if fermi is not None:
-            plt.ylabel(r"$E-E_f$ [eV]", fontsize=22)
+            ax1.set_ylabel(r"$E-E_f$ [eV]")
         else:
-            plt.ylabel(r"Energy [eV]", fontsize=22)
+            ax1.set_ylabel(r"Energy [eV]")
         if elimit:
-            plt.ylim(elimit)
+            ax1.set_ylim(elimit)
 
     elif mode == "parametric":
-        plot.parametricPlot(
+        fig, ax1 = plot.parametricPlot(
             cmap=cmap,
             vmin=vmin,
             vmax=vmax,
+            mask=mask,
             ticks=ticks,
             discontinuities=discontinuities,
+            ax=ax,
         )
         if fermi is not None:
-            plt.ylabel(r"$E-E_f$ [eV]", fontsize=22)
+            ax1.set_ylabel(r"$E-E_f$ [eV]")
         else:
-            plt.ylabel(r"Energy [eV]", fontsize=22)
+            ax1.set_ylabel(r"Energy [eV]")
         if elimit is not None:
-            plt.ylim(elimit)
+            ax1.set_ylim(elimit)
 
     elif mode == "atomic":
-        plot.atomicPlot(cmap=cmap, vmin=vmin, vmax=vmax)
+        fig, ax1 = plot.atomicPlot(cmap=cmap, vmin=vmin, vmax=vmax, ax=ax)
         if fermi is not None:
-            plt.ylabel(r"$E-E_f$ [eV]", fontsize=22)
+            ax1.set_ylabel(r"$E-E_f$ [eV]")
         else:
-            plt.ylabel(r"Energy [eV]", fontsize=22)
+            ax1.set_ylabel(r"Energy [eV]")
         if elimit is not None:
-            plt.ylim(elimit)
+            ax1.set_ylim(elimit)
     ###### end of mode dependent options ###########
-    plt.tight_layout()
+
     if grid:
-        plt.grid()
+        ax1.grid()
 
     if title:
-        plt.title(title, fontsize=22)
-
-    if exportplt:
-        return plt
+        ax1.set_title(title)
 
     else:
         if savefig:
             plt.savefig(savefig, bbox_inches="tight")
             plt.close()  # Added by Nicholas Pike to close memory issue of looping and creating many figures
+            return None, None
         else:
-            plt.show()
-        return
+            if show:
+                plt.show()
+            else:
+                pass
+        return fig, ax1
 
 
 # if __name__ == "__main__":
